﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import jp.progression.core.commands.Command;
	import jp.progression.events.CommandEvent;
	import jp.progression.events.ProcessEvent;
	import jp.progression.Progression;
	import jp.progression.scenes.SceneId;
	
	/*======================================================================*//**
	* コマンドの実行中に例外エラーが発生した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandCatchEvent.EXCEPTION
	*/
	[Event( name="exception", type="jp.progression.events.CommandCatchEvent" )]
	
	/*======================================================================*//**
	* Goto クラスは指定されたシーン識別子の指すシーンに移動するコマンドクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.commands.Goto;
	* import jp.progression.scenes.SceneId;
	* 
	* // Goto コマンドを実行する
	* new Goto( new SceneId( "/index/page1" ) )
	* 	.before( null, trace, [ "移動を開始します。" ] )
	* 	.after( null, trace, [ "移動を完了しました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class Goto extends Command {
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get enabled()	:Boolean { return _sceneId is SceneId ? super.enabled : false; }
		
		/*======================================================================*//**
		* 移動先を示すシーン識別子を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get sceneId()			:SceneId { return _sceneId; }
		public function set sceneId( value		:SceneId ):void { _sceneId = value; }
		private var _sceneId					:SceneId;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい Goto インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	sceneId		移動先を示すシーン識別子です。
		* @param	delay		処理開始まで遅延時間をミリ秒で指定します。
		*/
		public function Goto( sceneId:SceneId = null, delay:int = 0 ) {
			super( "Goto", delay );
			
			// 引数を設定する
			this.sceneId = sceneId;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// Progression インスタンスを取得する
			var prog:Progression = Progression.getInstanceBySceneId( _sceneId );
			
			// 存在しなければ終了する
			if ( !prog ) {
				// 処理を終了する
				_executeComplete();
				return;
			}
			
			// すでに処理をしていれば強制停止させる
			if ( prog.running ) {
				prog.addExclusivelyEventListener( ProcessEvent.PROCESS_INTERRUPT, _processInterrupt );
				prog.interrupt();
				return;
			}
			
			// 移動する
			prog.goto( _sceneId );
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* Goto インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい Goto インスタンスです。
		*/
		public override function clone():Command {
			var command:Command = new Goto( _sceneId ? _sceneId.clone() : null, delay );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
		
		
		
		
		
		/*======================================================================*//**
		* イベント処理が停止された場合に送出されます。
		*/
		private function _processInterrupt( e:ProcessEvent ):void {
			// イベントリスナーを解除する
			Progression( e.target ).completelyRemoveEventListener( ProcessEvent.PROCESS_INTERRUPT, _processInterrupt );
			
			// 移動する
			Progression( e.target ).goto( _sceneId );
		}
	}
}





