﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import flash.events.Event;
	import jp.nium.events.EventUnit;
	import jp.progression.core.commands.Command;
	import jp.progression.core.commands.CommandList;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	
	use namespace progression_internal;

	/*======================================================================*//**
	* ParallelList クラスは指定された複数のコマンドを全て同時に実行させる為のコマンドリストクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.commands.ParallelList;
	* import jp.progression.commands.Trace;
	* import jp.progression.commands.Wait;
	* 
	* // ParallelList コマンドを実行する
	* new ParallelList( 0,
	* 	new Wait( 1000 ),
	* 	new Trace( "出力します。" ) )
	* 	.before( null, trace, [ "コマンドを同時に実行します。" ] )
	* 	.after( null, trace, [ "コマンドを同時に実行しました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class ParallelList extends CommandList {
		
		/*======================================================================*//**
		* EventUnit インスタンスを取得します。 
		*/
		private var _eventUnit					:EventUnit;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい ParallelList インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	delay		処理開始まで遅延時間をミリ秒で指定します。
		* @param	commands	並列処理を実行したいコマンドオブジェクトを格納した配列です。
		*/
		public function ParallelList( delay:int = 0, ... commands:Array ) {
			super( "ParallelList", delay, commands );
			
			// 初期化する
			_eventUnit = new EventUnit();
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// コマンドが登録されていなければ終了する
			if ( commands.length == 0 ) {
				// 処理を終了する
				_executeFinally( new CommandEvent( CommandEvent.COMMAND_COMPLETE ) );
				return;
			}
			
			// 初期化する
			_eventUnit = new EventUnit();
			_eventUnit.addExclusivelyEventListener( Event.COMPLETE, _complete, false, int.MAX_VALUE, true );
			
			// イベントリスナーを登録する
			var l:int = commands.length;
			for ( var i:int = 0; i < l; i++ ) {
				var command:Command = Command( commands[i] );
				command.addExclusivelyEventListener( CommandCatchEvent.EXCEPTION, _catch, false, int.MAX_VALUE, true );
				command.addExclusivelyEventListener( CommandCatchEvent.ERROR, _catch, false, int.MAX_VALUE, true );
				_eventUnit.addEventDispatcher( command, CommandEvent.COMMAND_COMPLETE );
			}
			
			// 実行する
			for ( i = 0; i < l; i++ ) {
				Command( commands[i] ).execute( extra );
			}
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// 初期化する
			_eventUnit = new EventUnit();
			_eventUnit.addExclusivelyEventListener( Event.COMPLETE, _complete, false, int.MAX_VALUE, true );
			
			// コマンドが登録されていなければ
			if ( commands.length == 0 ) {
				// 停止処理を終了する
				_interruptComplete();
				return;
			}
			
			// イベントリスナーを登録する
			_eventUnit.addExclusivelyEventListener( Event.COMPLETE, _interrupt, false, int.MAX_VALUE, true );
			
			// 実行中のコマンドがあれば
			var l:int = commands.length;
			for ( var i:int = 0; i < l; i++ ) {
				var command:Command = Command( commands[i] );
				command.removeAllListeners( true );
				
				_eventUnit.addEventDispatcher( command, CommandEvent.COMMAND_INTERRUPT );
				command.interrupt( extra );
			}
		}
		
		/*======================================================================*//**
		* ParallelList インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい ParallelList インスタンスです。
		*/
		public override function clone():Command {
			var command:CommandList = new ParallelList( delay );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			
			// コマンドリストを複製する
			var l:int = commands.length;
			for ( var i:int = 0; i < l; i++ ) {
				command.addCommand( commands[i] );
			}
			
			return Command( command );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 登録された全てのイベントが発生した際に送出されます。
		*/
		private function _complete( e:Event ):void {
			// EventUnit のリスナーを解除する
			_eventUnit.removeAllListeners( true );
			
			// イベントリスナーを解除する
			var l:int = commands.length;
			for ( var i:int = 0; i < l; i++ ) {
				Command( commands[i] ).removeAllListeners( true );
			}
			
			// 処理を終了する
			_executeFinally( new CommandEvent( CommandEvent.COMMAND_COMPLETE ) );
		}
		
		/*======================================================================*//**
		* コマンドの処理を停止した場合に送出されます。
		*/
		private function _interrupt( e:Event ):void {
			// EventUnit のリスナーを解除する
			_eventUnit.removeAllListeners( true );
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* コマンド処理中にエラーが発生した場合に送出されます。
		*/
		private function _catch( e:CommandCatchEvent ):void {
			// EventUnit のリスナーを解除する
			_eventUnit.removeAllListeners( true );
			
			// イベントリスナーを解除する
			var l:int = commands.length;
			for ( var i:int = 0; i < l; i++ ) {
				Command( commands[i] ).removeAllListeners( true );
			}
			
			// 処理を終了する
			_executeCatch( e );
		}
	}
}





