﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import jp.progression.core.commands.Command;
	import jp.progression.core.commands.CommandList;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* SerialList クラスは指定された複数のコマンドを 1 つづつ順番に実行させる為のコマンドリストクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.commands.SerialList;
	* import jp.progression.commands.Trace;
	* import jp.progression.commands.Wait;
	* 
	* // SerialList コマンドを実行する
	* new SerialList( 0,
	* 	new Wait( 1000 ),
	* 	new Trace( "出力します。" ) )
	* 	.before( null, trace, [ "コマンドを連続で実行します。" ] )
	* 	.after( null, trace, [ "コマンドを連続で実行しました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class SerialList extends CommandList {
		
		/*======================================================================*//**
		* 
		*/
		private var _currentCount			:int = 0;
		
		/*======================================================================*//**
		* Command インスタンスを取得します。 
		*/
		private var _currentCommand			:Command;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい SerialList インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	delay		処理開始まで遅延時間をミリ秒で指定します。
		* @param	commands	直列処理を実行したいコマンドオブジェクトを格納した配列です。
		*/
		public function SerialList( delay:int = 0, ... commands:Array ) {
			super( "SerialList", delay, commands );
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// 処理ラインを初期化する
			_currentCount = 0;
			
			// 処理を実行する
			_execute();
		}
		
		/*======================================================================*//**
		* 
		*/
		private function _execute():void {
			// コマンドが登録されていなければ終了する
			if ( _currentCount >= commands.length ) {
				
				// 処理を終了する
				_executeFinally( new CommandEvent( CommandEvent.COMMAND_COMPLETE ) );
				return;
			}
			
			// 処理を実行する
			_currentCommand = Command( commands[_currentCount++] );
			_currentCommand.addExclusivelyEventListener( CommandEvent.COMMAND_COMPLETE, _commandComplete, false, int.MAX_VALUE, true );
			_currentCommand.addExclusivelyEventListener( CommandCatchEvent.EXCEPTION, _catch, false, int.MAX_VALUE, true );
			_currentCommand.addExclusivelyEventListener( CommandCatchEvent.ERROR, _catch, false, int.MAX_VALUE, true );
			_currentCommand.execute( extra );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// 実行中のコマンドがあれば
			if ( _currentCommand ) {
				// イベントリスナーを解除する
				_currentCommand.removeAllListeners( true );
				
				// 一つ前に戻る
				_currentCount--;
				
				// 停止する
				_interrupt();
				return;
			}
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* 
		*/
		private function _interrupt():void {
			// コマンドが登録されていなければ終了する
			if ( _currentCount >= commands.length ) {
				// 停止処理を終了する
				_interruptComplete();
				return;
			}
			
			// 処理を実行する
			_currentCommand = Command( commands[_currentCount++] );
			_currentCommand.addExclusivelyEventListener( CommandEvent.COMMAND_COMPLETE, _commandInterrupt, false, int.MAX_VALUE, true );
			_currentCommand.addExclusivelyEventListener( CommandEvent.COMMAND_INTERRUPT, _commandInterrupt, false, int.MAX_VALUE, true );
			_currentCommand.execute( extra );
			_currentCommand.interrupt( extra );
		}
		
		/*======================================================================*//**
		* SerialList インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい SerialList インスタンスです。
		*/
		public override function clone():Command {
			var command:CommandList = new SerialList( delay );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			
			// コマンドリストを複製する
			var l:int = commands.length;
			for ( var i:int = 0; i < l; i++ ) {
				command.addCommand( commands[i] );
			}
			
			return Command( command );
		}
		
		
		
		
		
		/*======================================================================*//**
		* コマンドの処理が完了した場合に送出されます。
		*/
		private function _commandComplete( e:CommandEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 処理を実行する
			_execute();
		}
		
		/*======================================================================*//**
		* コマンドの処理を停止した場合に送出されます。
		*/
		private function _commandInterrupt( e:CommandEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 処理を実行する
			_interrupt();
		}
		
		/*======================================================================*//**
		* コマンド処理中にエラーが発生した場合に送出されます。
		*/
		private function _catch( e:CommandCatchEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeCatch( e );
		}
	}
}





