﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	import jp.progression.core.commands.Command;
	import jp.progression.events.CommandEvent;
	
	/*======================================================================*//**
	* Wait クラスは指定された時間だけ処理を停止させるコマンドクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.commands.Wait;
	* 
	* // Wait コマンドを実行する
	* new Wait( 1000 )
	* 	.before( null, trace, [ "1 秒待ちます。" ] )
	* 	.after( null, trace, [ "1 秒待ちました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class Wait extends Command {
		
		/*======================================================================*//**
		* 処理を停止させたい時間をミリ秒で取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get time()			:int { return _time; }
		public function set time( value		:int ):void { _time = Math.max( 0, value ); }
		private var _time					:int = 0;
		
		/*======================================================================*//**
		* Timer インスタンスを取得します。 
		*/
		private var _timer					:Timer;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい Wait インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	time	処理を停止させたい時間のミリ秒です。
		* @param	delay	処理開始まで遅延時間をミリ秒で指定します。
		*/
		public function Wait( time:int = 0, delay:int = 0 ) {
			super( "Wait", delay );
			
			// 初期化する
			_timer = new Timer( 1, 1 );
			
			// 引数を設定する
			this.time = time;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// 遅延時間が 0 であれば終了する
			if ( _time == 0 ) {
				// 処理を終了する
				_executeComplete();
				return;
			}
			
			// Timer を初期化する
			_timer.reset();
			_timer.delay = _time;
			
			// イベントリスナーを登録する
			_timer.addEventListener( TimerEvent.TIMER_COMPLETE, _timerComplete, false, int.MAX_VALUE, true );
			
			// Timer を開始する
			_timer.start();
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// イベントリスナーを解除する
			_timer.removeEventListener( TimerEvent.TIMER_COMPLETE, _timerComplete );
			
			// タイマーを停止する
			_timer.stop();
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* Wait インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい Wait インスタンスです。
		*/
		public override function clone():Command {
			var command:Command = new Wait( _time, delay );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
		
		
		
		
		
		/*======================================================================*//**
		* Timer.repeatCount で設定された数の要求が完了するたびに送出されます。
		* */
		private function _timerComplete( e:TimerEvent ):void {
			Timer( e.target ).removeEventListener( TimerEvent.TIMER_COMPLETE, _timerComplete );
			
			// 処理を終了する
			_executeComplete();
		}
	}
}





