﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.core.commands {
	import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.utils.Dictionary;
	import flash.utils.getQualifiedClassName;
	import flash.utils.getQualifiedSuperclassName;
	import flash.utils.Timer;
	import jp.nium.events.EventIntegrator;
	import jp.nium.utils.ObjectUtil;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	import jp.progression.Progression;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* コマンドの処理が開始された場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandEvent.COMMAND_START
	*/
	[Event( name="commandStart", type="jp.progression.events.CommandEvent" )]
	
	/*======================================================================*//**
	* コマンドの処理が完了した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandEvent.COMMAND_COMPLETE
	*/
	[Event( name="commandComplete", type="jp.progression.events.CommandEvent" )]
	
	/*======================================================================*//**
	* コマンドの処理を停止した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandEvent.COMMAND_INTERRUPT
	*/
	[Event( name="commandInterrupt", type="jp.progression.events.CommandEvent" )]
	
	/*======================================================================*//**
	* Command クラスを直接インスタンス化することはできません。
	* new Command() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* package {
	* 	import jp.progression.core.commands.Command;
	* 	import jp.progression.events.CommandEvent;
	* 	
	* 	public class CustomCommand extends Command {
	* 		
	* 		public function CustomCommand( delay:int = 0 ) {
	* 			super( "CustomCommand", delay );
	* 		}
	* 		
	* 		protected override function _executeProgress():void {
	* 			// 任意の処理を実行させる
	* 			trace( "任意の処理を実行させます。" );
	* 			
	* 			// 処理を終了する
	* 			_executeComplete();
	* 		}
	* 		
	* 		public override function clone():Command {
	* 			return new CustomCommand( delay );
	* 		}
	* 		
	* 		public override function toString():String {
	* 			return "[object CustomCommand]";
	* 		}
	* 	}
	* }
	* </listing>
	*/
	public class Command extends EventIntegrator {
		
		/*======================================================================*//**
		* @private
		*/
		public static function get intermission()			:int { return _intermission; }
		public static function set intermission( value		:int ):void { _intermission = Math.max( 0, value ); }
		private static var _intermission					:int = 0;
		
		/*======================================================================*//**
		* @private
		*/
		public static function get thresholdDepth()			:int { return _thresholdDepth; }
		public static function set thresholdDepth( value	:int ):void { _thresholdDepth = Math.max( 0, value ); }
		private static var _thresholdDepth					:int = 5;
		
		/*======================================================================*//**
		* @private
		*/
		private static var _instanceLength					:int = 0;
		
		
		
		
		
		
		/*======================================================================*//**
		* コマンドの名前を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get name()							:String { return _name || _instanceId; }
		public function set name( value						:String ):void { _name = value; }
		private var _name									:String;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		private var _instanceId								:String;
		
		/*======================================================================*//**
		* コマンドの種類を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get type()							:String { return _type; }
		private var _type									:String;
		
		/*======================================================================*//**
		* コマンドが実行可能かどうかを取得または設定します。
		* ただし、すでに実行中の場合には設定を変更することはできません。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get enabled()						:Boolean { return _enabled; }
		public function set enabled( value					:Boolean ):void {
			if ( _running ) { return; }
			_enabled = value;
		}
		private var _enabled								:Boolean = true;
		
		/*======================================================================*//**
		* コマンドが実行中かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get running()						:Boolean { return _running; }
		private var _running								:Boolean = false;
		
		/*======================================================================*//**
		* コマンド処理の中断処理中かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get interrupting()					:Boolean { return _interrupting; }
		private var _interrupting							:Boolean = false;
		
		/*======================================================================*//**
		* コマンド実行時の遅延時間を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get delay()							:int { return _delay; }
		public function set delay( value					:int ):void { _delay = Math.max( 0, value ); }
		private var _delay									:int = 0;
		
		/*======================================================================*//**
		* execute() メソッド実行時に引数として指定されたオブジェクトを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get extra()							:Object { return _extra; }
		private var _extra									:Object;
		
		/*======================================================================*//**
		* コマンドが階層構造を持っている場合に、親に当たる Command インスタンスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get parent()						:Command { return progression_internal::parent; }
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get parent()			:Command { return _parent; }
		progression_internal function set parent( value		:Command ):void { _parent = value; }
		private var _parent									:Command;
		
		/*======================================================================*//**
		* コマンドが階層構造を持っている場合に、自身の階層深度を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get depth()							:int { return progression_internal::depth; }
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get depth()			:int { return _depth; }
		progression_internal function set depth( value		:int ):void { _depth = value; }
		private var _depth									:int = 0;
		
		/*======================================================================*//**
		* @private
		*/
		protected function get _beforeScorp()				:Object { return __beforeScorp; }
		private var __beforeScorp							:Object;
		
		/*======================================================================*//**
		* @private
		*/
		protected function get _beforeFunc()				:Function { return __beforeFunc; }
		private var __beforeFunc							:Function;
		
		/*======================================================================*//**
		* @private
		*/
		protected function get _beforeArgs()				:Array { return __beforeArgs; }
		private var __beforeArgs							:Array;
		
		/*======================================================================*//**
		* @private
		*/
		protected function get _afterScorp()				:Object { return __afterScorp; }
		private var __afterScorp							:Object;
		
		/*======================================================================*//**
		* @private
		*/
		protected function get _afterFunc()					:Function { return __afterFunc; }
		private var __afterFunc								:Function;
		
		/*======================================================================*//**
		* @private
		*/
		protected function get _afterArgs()					:Array { return __afterArgs; }
		private var __afterArgs								:Array;
		
		/*======================================================================*//**
		* Timer インスタンスを取得します。 
		*/
		private var _timer									:Timer;
		
		/*======================================================================*//**
		* クラス名を取得します。
		*/
		private var _className								:String;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい Command インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	type			コマンドの種類です。
		* @param	delay			コマンド実行時の遅延時間です。
		* @throws	ArgumentError	Command クラスを直接インスタンス化することはできません。
		*/
		public function Command( type:String, delay:int = 0 ) {
			// 継承せずにオブジェクト化されたらエラーを送出する
			if ( getQualifiedSuperclassName( this ) == getQualifiedClassName( EventIntegrator ) ) { throw new ArgumentError( "Error #2012: Command クラスを直接インスタンス化することはできません。" ); }
			
			// 引数を設定する
			_type = type;
			this.delay = delay;
			
			// 初期化する
			_className = getQualifiedClassName( this ).split( "::" ).pop();
			_instanceId = "command_" + _instanceLength++;
			_extra = {};
			_timer = new Timer( 1, 1 );
			__beforeArgs = [];
			__afterArgs = [];
		}
		
		
		
		
		
		/*======================================================================*//**
		* コマンドを実行します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	extra		実行時に渡したい任意のオブジェクトです。
		*/
		public function execute( extra:Object = null ):void {
			// 無効化されていたら終了する
			if ( !_enabled ) { return; }
			
			// 実行中なら終了する
			if ( _running ) { return; }
			
			_extra = extra || {};
			
			// 処理を開始する
			_running = true;
			
			//　遅延時間を取得する
			var delay:int = _intermission + _delay;
			
			// 限界深度を超えていれば遅延を発生させる
			if ( _depth > _thresholdDepth ) {
				delay = Math.max( 1, delay );
			}
			
			// 遅延時間の設定が存在しなければ処理を開始する
			if ( delay == 0 ) {
				_executeStart();
				return;
			}
			
			// Timer を初期化する
			_timer.reset();
			_timer.delay = delay;
			
			// イベントリスナーを登録する
			_timer.addEventListener( TimerEvent.TIMER_COMPLETE, _timerComplete, false, int.MAX_VALUE, true );
			
			// Timer を開始する
			_timer.start();
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected function _executeStart():void {
			Progression.verbose.log( this, null, _depth + 2, "<" + type + " : " + name + "> コマンドを実行" );
			
			// イベントを送出する
			dispatchEvent( new CommandEvent( CommandEvent.COMMAND_START ) );
			
			// 事前処理を実行する
			if ( _beforeFunc is Function ) {
				try {
					_beforeFunc.apply( _beforeScorp, _beforeArgs );
				}
				catch ( e:Error ) {
					_running = false;
					dispatchEvent( new CommandCatchEvent( CommandCatchEvent.EXCEPTION, false, false, this, e ) );
					return;
				}
			}
			
			try {
				// 処理を実行する
				_executeProgress();
			}
			catch ( e:Error ) {
				_running = false;
				dispatchEvent( new CommandCatchEvent( CommandCatchEvent.EXCEPTION, false, false, this, e ) );
				return;
			}
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected function _executeProgress():void {
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected function _executeComplete():void {
			// 事後処理を実行する
			if ( _afterFunc is Function ) {
				try {
					_afterFunc.apply( _afterScorp, _afterArgs );
				}
				catch ( e:Error ) {
					_running = false;
					dispatchEvent( new CommandCatchEvent( CommandCatchEvent.EXCEPTION, false, false, this, e ) );
					return;
				}
			}
			
			// 処理を終了する
			_running = false;
			
			// イベントを送出する
			dispatchEvent( new CommandEvent( CommandEvent.COMMAND_COMPLETE ) );
		}
		
		/*======================================================================*//**
		* コマンド処理を停止します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function interrupt( extra:Object = null ):void {
			_extra = extra || {};
			
			// 停止処理中なら終了する
			if ( _interrupting ) { return; }
			
			// 停止処理を開始する
			_running = true;
			_interrupting = true;
			
			// 停止処理を開始する
			_interruptStart()
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected function _interruptStart():void {
			Progression.verbose.log( this, null, _depth + 2, "<" + type + " : " + name + "> コマンドを停止" );
			
			// 遅延中であれば
			if ( _timer.running ) {
				// イベントリスナーを解除する
				_timer.removeEventListener( TimerEvent.TIMER_COMPLETE, _timerComplete );
				
				// タイマーを停止する
				_timer.stop();
				
				// 停止処理を終了する
				_interruptComplete();
				return;
			}
			
			// 実行中であれば
			if ( _running ) {
				try {
					// 停止処理を開始する
					_interruptProgress();
				}
				catch ( e:Error ) {
					_running = false;
					_interrupting = false;
					dispatchEvent( new CommandCatchEvent( CommandCatchEvent.EXCEPTION, false, false, this, e ) );
					return;
				}
				return;
			}
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected function _interruptProgress():void {
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected function _interruptComplete():void {
			// 処理を終了する
			_running = false;
			_interrupting = false;
			
			// イベントを送出する
			dispatchEvent( new CommandEvent( CommandEvent.COMMAND_INTERRUPT ) );
		}
		
		/*======================================================================*//**
		* コマンドの実行直前に処理させたい関数を設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	scope	関数の適用先のオブジェクトです。
		* @param	func	実行させたい関数です。
		* @param	args	関数実行時に引数として使用したい配列です。
		* @return			自身の参照です。
		*/
		public function before( scope:Object, func:Function, ... args:Array ):Command {
			__beforeScorp = scope || this;
			__beforeFunc = func;
			__beforeArgs = args ? args.concat() : [];
			return this;
		}
		
		/*======================================================================*//**
		* コマンドの実行完了直後に処理させたい関数を設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	scope	関数の適用先のオブジェクトです。
		* @param	func	実行させたい関数です。
		* @param	args	関数実行時に引数として使用したい配列です。
		* @return			自身の参照です。
		*/
		public function after( scope:Object, func:Function, ... args:Array ):Command {
			__afterScorp = scope || this;
			__afterFunc = func;
			__afterArgs = args ? args.concat() : [];
			return this;
		}
		
		/*======================================================================*//**
		* コマンドに対してすぐに関数を実行します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	scope	関数の適用先のオブジェクトです。
		* @param	func	実行させたい関数です。
		* @param	args	関数実行時に引数として使用したい配列です。
		* @return			自身の参照です。
		*/
		public function apply( scope:Object, func:Function, ... args:Array ):Command {
			func.apply( scope, args );
			return this;
		}
		
		/*======================================================================*//**
		* Command インスタンスに対して、複数のプロパティを一括設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	props	設定したいプロパティを含んだ Object です。
		* @return			自身の参照です。
		*/
		public function setProperties( props:Object ):Command {
			ObjectUtil.setProperties( this, props );
			return this;
		}
		
		/*======================================================================*//**
		* Command インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい Command インスタンスです。
		*/
		public function clone():Command {
			var command:Command = new Command( _type, _delay );
			command.enabled = _enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object " + _className + " : " + name + "]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* Timer.repeatCount で設定された数の要求が完了するたびに送出されます。
		*/
		private function _timerComplete( e:TimerEvent ):void {
			Timer( e.target ).removeEventListener( TimerEvent.TIMER_COMPLETE, _timerComplete );
			
			// 処理を開始する
			_executeStart();
		}
	}
}





