﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.scenes {
	
	/*======================================================================*//**
	* SceneId クラスは、Progression インスタンスが管理するシーンオブジェクト構造上の特定のシーンを表すデータクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.scenes.SceneId;
	* 
	* var sceneId:SceneId = new SceneId( "/index" );
	* trace( sceneId.toString() );
	* trace( sceneId = sceneId.transfer( "page1" ) );
	* trace( sceneId = sceneId.transfer( "page2" ) );
	* trace( sceneId = sceneId.transfer( "../" ) );
	* trace( sceneId = sceneId.transfer( "../page3" ) );
	* </listing>
	*/
	public class SceneId {
		
		/*======================================================================*//**
		* シーンパスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get path()			:String { return _path; }
		private var _path					:String;
		
		/*======================================================================*//**
		* シーンパスの深度を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get length()		:int { return path.split( "/" ).length - 1; }
		
		
		
		
		
		/*======================================================================*//**
		* 新しい SceneId インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	scenePath		シーン識別子に変換する String インスタンスです。
		* @throws	ArgumentError	指定されたシーンパスの書式が正しくありません。
		*/
		public function SceneId( scenePath:String ) {
			// 書式が正しくない場合エラーを送出する
			if ( !validate( scenePath ) ) { throw new ArgumentError( "指定されたシーンパスの書式が正しくありません。" ); }
			
			// 引数を設定する
			_path = scenePath;
		}
		
		
		
		
		
		/*======================================================================*//**
		* シーンパスの書式が正しいかどうかを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	scenePath	書式を調べるシーンパスです。
		* @return				書式が正しければ true に、それ以外の場合は false になります。
		*/
		public static function validate( scenePath:String ):Boolean {
			return new RegExp( "^(/[-_A-Za-z0-9]+(/.|/..)*)*(/[-_A-Za-z0-9]+)+$" ).test( scenePath );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定された絶対シーンパスもしくは相対シーンパスを使用して移動後のシーン識別子を返します。
		* 元のシーン識別子は変更されません。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	scenePath		移動先のシーンパスです。
		* @return					移動後のシーン識別子です。
		* @throws	ArgumentError	指定されたシーンパスの書式が正しくありません。
		*/
		public function transfer( scenePath:String ):SceneId {
			// 相対パスの場合、既存のパスに結合する
			if ( scenePath.charAt(0) != "/" ) {
				scenePath = path + "/" + scenePath;
			}
			
			// パスに /./ が存在すれば結合する
			scenePath = scenePath.replace( "/./", "/" );
			
			// /A/B/../ なら /A/ に変換する
			var pattern:String = "/[-_A-Za-z0-9]+/[-_A-Za-z0-9]+/../";
			while ( new RegExp( pattern, "g" ).test( scenePath ) ) {
				scenePath = scenePath.replace( new RegExp( pattern, "g" ), function():String {
					return String( arguments[0] ).split( "/" ).slice( 0, 2 ).join( "/" ) + "/";
					} );
			}
			
			// /../ が存在すれば削除する
			scenePath = scenePath.replace( new RegExp( "/(../)*$", "g" ), "" );
			
			// 書式が正しくない場合エラーを送出する
			if ( !validate( scenePath ) ) { throw new ArgumentError( "指定されたシーンパスの書式が正しくありません。" ); }
			
			return new SceneId( scenePath );
		}
		
		/*======================================================================*//**
		* シーン識別子の格納するシーンパスの指定された範囲のエレメントを取り出して、新しいシーン識別子を返します。
		* 元のシーン識別子は変更されません。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	startIndex	スライスの始点のインデックスを示す数値です。
		* @param	endIndex	スライスの終点のインデックスを示す数値です。このパラメータを省略すると、スライスには配列の最初から最後までのすべてのエレメントが取り込まれます。endIndex が負の数値の場合、終点は配列の末尾から開始します。つまり、-1 が最後のエレメントです。
		* @return				元のシーンパスから取り出した一連のエレメントから成るシーン識別子です。
		*/
		public function slice( startIndex:int = 0, endIndex:int = 16777215 ):SceneId {
			endIndex = Math.max( 0, endIndex );
			startIndex = Math.min( startIndex, endIndex );
			
			var dirList:Array = path.split( "/" );
			dirList = dirList.splice( startIndex + 1, endIndex + 1 );
			
			return new SceneId( "/" + dirList.join( "/" ) );
		}
		
		/*======================================================================*//**
		* 指定されたシーン識別子が、自身の表すシーンパスの子シーンオブジェクトを指しているかどうかを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	sceneId		テストする子シーン識別子です。
		* @return				子シーンオブジェクトを指していれば true に、それ以外の場合は false になります。
		*/
		public function contains( sceneId:SceneId ):Boolean {
			var path1:Array = path.split( "/" );
			var path2:Array = sceneId.path.split( "/" );
			
			var l:int = path1.length;
			for ( var i:int = 0; i < l; i++ ) {
				if ( path1[i] != path2[i] ) { return false; }
			}
			return true;
		}
		
		/*======================================================================*//**
		* 指定されたシーン識別子が、自身の表すシーンパスと同一かどうかを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	sceneId		テストする子シーン識別子です。
		* @return				同一のシーンパスを指していれば true に、それ以外の場合は false になります。
		*/
		public function equals( sceneId:SceneId ):Boolean {
			return path == sceneId.path;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public function equal( sceneId:SceneId ):Boolean {
			throw new ArgumentError( "SceneId クラスの equal() メソッドは ActionScript との互換性を考慮して equals() メソッドに変更されました。\n同様の操作をする場合、今後は equals() メソッドを使用してください。" );
		}
		
		/*======================================================================*//**
		* 指定位置にあるシーンの名前を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	index	取得した名前のあるシーンの位置です。
		* @return			指定位置にあるシーンの名前です。
		*/
		public function getNameByIndex( index:int ):String {
			var dirList:Array = path.split( "/" );
			dirList.shift();
			
			// マイナスが指定されたら、最後尾からのインデックスを取得する
			if ( index < 0 ) {
				index += dirList.length;
			}
			
			return dirList[index];
		}
		
		/*======================================================================*//**
		* SceneId インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい SceneId インスタンスです。
		*/
		public function clone():SceneId {
			return new SceneId( path );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public function toString():String {
			return path;
		}
	}
}





